@@ -50,11 +50,13 @@ u'<html></html>'
 """
 import re
 from inspect import getargspec
+
 from django.conf import settings
 from django.template.context import Context, RequestContext, ContextPopException
+from django.utils.importlib import import_module
 from django.utils.itercompat import is_iterable
 from django.utils.functional import curry, Promise
-from django.utils.text import smart_split
+from django.utils.text import smart_split, unescape_string_literal
 from django.utils.encoding import smart_unicode, force_unicode, smart_str
 from django.utils.translation import ugettext as _
 from django.utils.safestring import SafeData, EscapeData, mark_safe, mark_for_escaping
@@ -101,20 +103,7 @@ builtins = []
 invalid_var_format_string = None
 
 class TemplateSyntaxError(Exception):
-    def __str__(self):
-        try:
-            import cStringIO as StringIO
-        except ImportError:
-            import StringIO
-        output = StringIO.StringIO()
-        output.write(Exception.__str__(self))
-        # Check if we wrapped an exception and print that too.
-        if hasattr(self, 'exc_info'):
-            import traceback
-            output.write('\n\nOriginal ')
-            e = self.exc_info
-            traceback.print_exception(e[0], e[1], e[2], 500, output)
-        return output.getvalue()
+    pass
 
 class TemplateDoesNotExist(Exception):
     pass
@@ -409,6 +398,20 @@ class TokenParser(object):
         "A microparser that parses for a value: some string constant or variable name."
         subject = self.subject
         i = self.pointer
+
+        def next_space_index(subject, i):
+            "Increment pointer until a real space (i.e. a space not within quotes) is encountered"
+            while i < len(subject) and subject[i] not in (' ', '\t'):
+                if subject[i] in ('"', "'"):
+                    c = subject[i]
+                    i += 1
+                    while i < len(subject) and subject[i] != c:
+                        i += 1
+                    if i >= len(subject):
+                        raise TemplateSyntaxError("Searching for value. Unexpected end of string in column %d: %s" % (i, subject))
+                i += 1
+            return i
+
         if i >= len(subject):
             raise TemplateSyntaxError("Searching for value. Expected another value but found end of string: %s" % subject)
         if subject[i] in ('"', "'"):
@@ -419,6 +422,10 @@ class TokenParser(object):
             if i >= len(subject):
                 raise TemplateSyntaxError("Searching for value. Unexpected end of string in column %d: %s" % (i, subject))
             i += 1
+
+            # Continue parsing until next "real" space, so that filters are also included
+            i = next_space_index(subject, i)
+
             res = subject[p:i]
             while i < len(subject) and subject[i] in (' ', '\t'):
                 i += 1
@@ -427,15 +434,7 @@ class TokenParser(object):
             return res
         else:
             p = i
-            while i < len(subject) and subject[i] not in (' ', '\t'):
-                if subject[i] in ('"', "'"):
-                    c = subject[i]
-                    i += 1
-                    while i < len(subject) and subject[i] != c:
-                        i += 1
-                    if i >= len(subject):
-                        raise TemplateSyntaxError("Searching for value. Unexpected end of string in column %d: %s" % (i, subject))
-                i += 1
+            i = next_space_index(subject, i)
             s = subject[p:i]
             while i < len(subject) and subject[i] in (' ', '\t'):
                 i += 1
@@ -443,32 +442,45 @@ class TokenParser(object):
             self.pointer = i
             return s
 
+# This only matches constant *strings* (things in quotes or marked for
+# translation). Numbers are treated as variables for implementation reasons
+# (so that they retain their type when passed to filters).
+constant_string = r"""
+(?:%(i18n_open)s%(strdq)s%(i18n_close)s|
+%(i18n_open)s%(strsq)s%(i18n_close)s|
+%(strdq)s|
+%(strsq)s)
+""" % {
+    'strdq': r'"[^"\\]*(?:\\.[^"\\]*)*"', # double-quoted string
+    'strsq': r"'[^'\\]*(?:\\.[^'\\]*)*'", # single-quoted string
+    'i18n_open' : re.escape("_("),
+    'i18n_close' : re.escape(")"),
+    }
+constant_string = constant_string.replace("\n", "")
+
 filter_raw_string = r"""
-^%(i18n_open)s"(?P<i18n_constant>%(str)s)"%(i18n_close)s|
-^"(?P<constant>%(str)s)"|
-^(?P<var>[%(var_chars)s]+)|
+^(?P<constant>%(constant)s)|
+^(?P<var>[%(var_chars)s]+|%(num)s)|
  (?:%(filter_sep)s
      (?P<filter_name>\w+)
          (?:%(arg_sep)s
              (?:
-              %(i18n_open)s"(?P<i18n_arg>%(str)s)"%(i18n_close)s|
-              "(?P<constant_arg>%(str)s)"|
-              (?P<var_arg>[%(var_chars)s]+)
+              (?P<constant_arg>%(constant)s)|
+              (?P<var_arg>[%(var_chars)s]+|%(num)s)
              )
          )?
  )""" % {
-    'str': r"""[^"\\]*(?:\\.[^"\\]*)*""",
+    'constant': constant_string,
+    'num': r'[-+\.]?\d[\d\.e]*',
     'var_chars': "\w\." ,
     'filter_sep': re.escape(FILTER_SEPARATOR),
     'arg_sep': re.escape(FILTER_ARGUMENT_SEPARATOR),
-    'i18n_open' : re.escape("_("),
-    'i18n_close' : re.escape(")"),
   }
 
 filter_re = re.compile(filter_raw_string, re.UNICODE|re.VERBOSE)
 
 class FilterExpression(object):
-    """
+    r"""
     Parses a variable token and its optional filters (all as a single string),
     and return a list of tuples of the filter name and arguments.
     Sample:
@@ -486,65 +498,62 @@ class FilterExpression(object):
     def __init__(self, token, parser):
         self.token = token
         matches = filter_re.finditer(token)
-        var = None
+        var_obj = None
         filters = []
         upto = 0
         for match in matches:
             start = match.start()
             if upto != start:
                 raise TemplateSyntaxError("Could not parse some characters: %s|%s|%s"  % \
-                                           (token[:upto], token[upto:start], token[start:]))
-            if var == None:
-                var, constant, i18n_constant = match.group("var", "constant", "i18n_constant")
-                if i18n_constant is not None:
-                    # Don't pass the empty string to gettext, because the empty
-                    # string translates to meta information.
-                    if i18n_constant == "":
-                        var = '""'
-                    else:
-                        var = '"%s"' %  _(i18n_constant.replace(r'\"', '"'))
-                elif constant is not None:
-                    var = '"%s"' % constant.replace(r'\"', '"')
-                upto = match.end()
-                if var == None:
-                    raise TemplateSyntaxError("Could not find variable at start of %s" % token)
-                elif var.find(VARIABLE_ATTRIBUTE_SEPARATOR + '_') > -1 or var[0] == '_':
-                    raise TemplateSyntaxError("Variables and attributes may not begin with underscores: '%s'" % var)
+                        (token[:upto], token[upto:start], token[start:]))
+            if var_obj is None:
+                var, constant = match.group("var", "constant")
+                if constant:
+                    try:
+                        var_obj = Variable(constant).resolve({})
+                    except VariableDoesNotExist:
+                        var_obj = None
+                elif var is None:
+                    raise TemplateSyntaxError("Could not find variable at start of %s." % token)
+                else:
+                    var_obj = Variable(var)
             else:
                 filter_name = match.group("filter_name")
                 args = []
-                constant_arg, i18n_arg, var_arg = match.group("constant_arg", "i18n_arg", "var_arg")
-                if i18n_arg:
-                    args.append((False, _(i18n_arg.replace(r'\"', '"'))))
-                elif constant_arg is not None:
-                    args.append((False, constant_arg.replace(r'\"', '"')))
+                constant_arg, var_arg = match.group("constant_arg", "var_arg")
+                if constant_arg:
+                    args.append((False, Variable(constant_arg).resolve({})))
                 elif var_arg:
                     args.append((True, Variable(var_arg)))
                 filter_func = parser.find_filter(filter_name)
                 self.args_check(filter_name,filter_func, args)
                 filters.append( (filter_func,args))
-                upto = match.end()
+            upto = match.end()
         if upto != len(token):
             raise TemplateSyntaxError("Could not parse the remainder: '%s' from '%s'" % (token[upto:], token))
+
         self.filters = filters
-        self.var = Variable(var)
+        self.var = var_obj
 
     def resolve(self, context, ignore_failures=False):
-        try:
-            obj = self.var.resolve(context)
-        except VariableDoesNotExist:
-            if ignore_failures:
-                obj = None
-            else:
-                if settings.TEMPLATE_STRING_IF_INVALID:
-                    global invalid_var_format_string
-                    if invalid_var_format_string is None:
-                        invalid_var_format_string = '%s' in settings.TEMPLATE_STRING_IF_INVALID
-                    if invalid_var_format_string:
-                        return settings.TEMPLATE_STRING_IF_INVALID % self.var
-                    return settings.TEMPLATE_STRING_IF_INVALID
+        if isinstance(self.var, Variable):
+            try:
+                obj = self.var.resolve(context)
+            except VariableDoesNotExist:
+                if ignore_failures:
+                    obj = None
                 else:
-                    obj = settings.TEMPLATE_STRING_IF_INVALID
+                    if settings.TEMPLATE_STRING_IF_INVALID:
+                        global invalid_var_format_string
+                        if invalid_var_format_string is None:
+                            invalid_var_format_string = '%s' in settings.TEMPLATE_STRING_IF_INVALID
+                        if invalid_var_format_string:
+                            return settings.TEMPLATE_STRING_IF_INVALID % self.var
+                        return settings.TEMPLATE_STRING_IF_INVALID
+                    else:
+                        obj = settings.TEMPLATE_STRING_IF_INVALID
+        else:
+            obj = self.var
         for func, args in self.filters:
             arg_vals = []
             for lookup, arg in args:
@@ -609,7 +618,7 @@ def resolve_variable(path, context):
     return Variable(path).resolve(context)
 
 class Variable(object):
-    """
+    r"""
     A template variable, resolvable against a given context. The variable may be
     a hard-coded string (if it begins and ends with single or double quote
     marks)::
@@ -623,8 +632,6 @@ class Variable(object):
         >>> c = AClass()
         >>> c.article = AClass()
         >>> c.article.section = u'News'
-        >>> Variable('article.section').resolve(c)
-        u'News'
 
     (The example assumes VARIABLE_ATTRIBUTE_SEPARATOR is '.')
     """
@@ -661,11 +668,13 @@ class Variable(object):
                 var = var[2:-1]
             # If it's wrapped with quotes (single or double), then
             # we're also dealing with a literal.
-            if var[0] in "\"'" and var[0] == var[-1]:
-                self.literal = mark_safe(var[1:-1])
-            else:
+            try:
+                self.literal = mark_safe(unescape_string_literal(var))
+            except ValueError:
                 # Otherwise we'll set self.lookups so that resolve() knows we're
                 # dealing with a bonafide variable
+                if var.find(VARIABLE_ATTRIBUTE_SEPARATOR + '_') > -1 or var[0] == '_':
+                    raise TemplateSyntaxError("Variables and attributes may not begin with underscores: '%s'" % var)
                 self.lookups = tuple(var.split(VARIABLE_ATTRIBUTE_SEPARATOR))
 
     def resolve(self, context):
@@ -731,6 +740,11 @@ class Variable(object):
                         current = settings.TEMPLATE_STRING_IF_INVALID
                     else:
                         raise
+            except Exception, e:
+                if getattr(e, 'silent_variable_failure', False):
+                    current = settings.TEMPLATE_STRING_IF_INVALID
+                else:
+                    raise
 
         return current
 
@@ -738,6 +752,7 @@ class Node(object):
     # Set this to True for nodes that must be first in the template (although
     # they can be preceded by text nodes.
     must_be_first = False
+    child_nodelists = ('nodelist',)
 
     def render(self, context):
         "Return the node rendered as a string"
@@ -751,8 +766,10 @@ class Node(object):
         nodes = []
         if isinstance(self, nodetype):
             nodes.append(self)
-        if hasattr(self, 'nodelist'):
-            nodes.extend(self.nodelist.get_nodes_by_type(nodetype))
+        for attr in self.child_nodelists:
+            nodelist = getattr(self, attr, None)
+            if nodelist:
+                nodes.extend(nodelist.get_nodes_by_type(nodetype))
         return nodes
 
 class NodeList(list):
@@ -789,7 +806,7 @@ class TextNode(Node):
 
     def render(self, context):
         return self.s
-    
+
 def _render_value_in_context(value, context):
     """
     Converts any value to a string to become part of a rendered template. This
@@ -943,7 +960,7 @@ def get_library(module_name):
     lib = libraries.get(module_name, None)
     if not lib:
         try:
-            mod = __import__(module_name, {}, {}, [''])
+            mod = import_module(module_name)
         except ImportError, e:
             raise InvalidTemplateLibrary("Could not load template library from %s, %s" % (module_name, e))
         try:
