@@ -98,25 +98,53 @@ class TextFile(models.Model):
     def __unicode__(self):
         return self.description
 
-class ImageFile(models.Model):
-    def custom_upload_path(self, filename):
-        path = self.path or 'tests'
-        return '%s/%s' % (path, filename)
-
-    description = models.CharField(max_length=20)
+try:
+    # If PIL is available, try testing ImageFields. Checking for the existence
+    # of Image is enough for CPython, but for PyPy, you need to check for the
+    # underlying modules If PIL is not available, ImageField tests are omitted.
+    # Try to import PIL in either of the two ways it can end up installed.
     try:
-        # If PIL is available, try testing PIL.
-        # Checking for the existence of Image is enough for CPython, but
-        # for PyPy, you need to check for the underlying modules
-        # If PIL is not available, this test is equivalent to TextFile above.
         from PIL import Image, _imaging
-        image = models.ImageField(storage=temp_storage, upload_to=custom_upload_path)
     except ImportError:
-        image = models.FileField(storage=temp_storage, upload_to=custom_upload_path)
-    path = models.CharField(max_length=16, blank=True, default='')
+        import Image, _imaging
+    
+    test_images = True
 
-    def __unicode__(self):
-        return self.description
+    class ImageFile(models.Model):
+        def custom_upload_path(self, filename):
+            path = self.path or 'tests'
+            return '%s/%s' % (path, filename)
+
+        description = models.CharField(max_length=20)
+
+        # Deliberately put the image field *after* the width/height fields to
+        # trigger the bug in #10404 with width/height not getting assigned.
+        width = models.IntegerField(editable=False)
+        height = models.IntegerField(editable=False)
+        image = models.ImageField(storage=temp_storage, upload_to=custom_upload_path,
+                                  width_field='width', height_field='height')
+        path = models.CharField(max_length=16, blank=True, default='')
+
+        def __unicode__(self):
+            return self.description
+
+    class OptionalImageFile(models.Model):
+        def custom_upload_path(self, filename):
+            path = self.path or 'tests'
+            return '%s/%s' % (path, filename)
+
+        description = models.CharField(max_length=20)
+        image = models.ImageField(storage=temp_storage, upload_to=custom_upload_path,
+                                  width_field='width', height_field='height',
+                                  blank=True, null=True)
+        width = models.IntegerField(editable=False, null=True)
+        height = models.IntegerField(editable=False, null=True)
+        path = models.CharField(max_length=16, blank=True, default='')
+
+        def __unicode__(self):
+            return self.description
+except ImportError:
+    test_images = False
 
 class CommaSeparatedInteger(models.Model):
     field = models.CommaSeparatedIntegerField(max_length=20)
@@ -177,6 +205,22 @@ class Post(models.Model):
     def __unicode__(self):
         return self.name
 
+class MarkupField(models.CharField):
+    def __init__(self, *args, **kwargs):
+        kwargs["max_length"] = 20
+        super(MarkupField, self).__init__(*args, **kwargs)
+
+    def formfield(self, **kwargs):
+        # don't allow this field to be used in form (real use-case might be
+        # that you know the markup will always be X, but it is among an app
+        # that allows the user to say it could be something else)
+        # regressed at r10062
+        return None
+
+class CustomFieldForExclusionModel(models.Model):
+    name = models.CharField(max_length=10)
+    markup = MarkupField()
+
 __test__ = {'API_TESTS': """
 >>> from django import forms
 >>> from django.forms.models import ModelForm, model_to_dict
@@ -319,6 +363,29 @@ We can also subclass the Meta inner class to change the fields list.
 <tr><th><label for="id_slug">Slug:</label></th><td><input id="id_slug" type="text" name="slug" maxlength="20" /></td></tr>
 <tr><th><label for="id_checkbox">Checkbox:</label></th><td><input type="checkbox" name="checkbox" id="id_checkbox" /></td></tr>
 
+# test using fields to provide ordering to the fields
+>>> class CategoryForm(ModelForm):
+...     class Meta:
+...         model = Category
+...         fields = ['url', 'name']
+
+>>> CategoryForm.base_fields.keys()
+['url', 'name']
+
+
+>>> print CategoryForm()
+<tr><th><label for="id_url">The URL:</label></th><td><input id="id_url" type="text" name="url" maxlength="40" /></td></tr>
+<tr><th><label for="id_name">Name:</label></th><td><input id="id_name" type="text" name="name" maxlength="20" /></td></tr>
+
+>>> class CategoryForm(ModelForm):
+...     class Meta:
+...         model = Category
+...         fields = ['slug', 'url', 'name']
+...         exclude = ['url']
+
+>>> CategoryForm.base_fields.keys()
+['slug', 'name']
+
 # Old form_for_x tests #######################################################
 
 >>> from django.forms import ModelForm, CharField
@@ -1072,6 +1139,18 @@ True
 >>> instance.file
 <FieldFile: tests/test3.txt>
 
+# Instance can be edited w/out re-uploading the file and existing file should be preserved.
+
+>>> f = TextFileForm(data={'description': u'New Description'}, instance=instance)
+>>> f.fields['file'].required = False
+>>> f.is_valid()
+True
+>>> instance = f.save()
+>>> instance.description
+u'New Description'
+>>> instance.file
+<FieldFile: tests/test3.txt>
+
 # Delete the current file since this is not done by Django.
 >>> instance.file.delete()
 >>> instance.delete()
@@ -1086,7 +1165,10 @@ True
 # Delete the current file since this is not done by Django.
 >>> instance.file.delete()
 >>> instance.delete()
+"""}
 
+if test_images:
+    __test__['API_TESTS'] += """
 # ImageField ###################################################################
 
 # ImageField and FileField are nearly identical, but they differ slighty when
@@ -1098,6 +1180,7 @@ True
 ...         model = ImageFile
 
 >>> image_data = open(os.path.join(os.path.dirname(__file__), "test.png"), 'rb').read()
+>>> image_data2 = open(os.path.join(os.path.dirname(__file__), "test2.png"), 'rb').read()
 
 >>> f = ImageFileForm(data={'description': u'An image'}, files={'image': SimpleUploadedFile('test.png', image_data)})
 >>> f.is_valid()
@@ -1107,9 +1190,14 @@ True
 >>> instance = f.save()
 >>> instance.image
 <...FieldFile: tests/test.png>
+>>> instance.width
+16
+>>> instance.height
+16
 
-# Delete the current file since this is not done by Django.
->>> instance.image.delete()
+# Delete the current file since this is not done by Django, but don't save
+# because the dimension fields are not null=True.
+>>> instance.image.delete(save=False)
 
 >>> f = ImageFileForm(data={'description': u'An image'}, files={'image': SimpleUploadedFile('test.png', image_data)})
 >>> f.is_valid()
@@ -1119,8 +1207,12 @@ True
 >>> instance = f.save()
 >>> instance.image
 <...FieldFile: tests/test.png>
+>>> instance.width
+16
+>>> instance.height
+16
 
-# Edit an instance that already has the image defined in the model. This will not
+# Edit an instance that already has the (required) image defined in the model. This will not
 # save the image again, but leave it exactly as it is.
 
 >>> f = ImageFileForm(data={'description': u'Look, it changed'}, instance=instance)
@@ -1131,62 +1223,103 @@ True
 >>> instance = f.save()
 >>> instance.image
 <...FieldFile: tests/test.png>
+>>> instance.height
+16
+>>> instance.width
+16
 
-# Delete the current image since this is not done by Django.
-
->>> instance.image.delete()
+# Delete the current file since this is not done by Django, but don't save
+# because the dimension fields are not null=True.
+>>> instance.image.delete(save=False)
 
 # Override the file by uploading a new one.
 
->>> f = ImageFileForm(data={'description': u'Changed it'}, files={'image': SimpleUploadedFile('test2.png', image_data)}, instance=instance)
+>>> f = ImageFileForm(data={'description': u'Changed it'}, files={'image': SimpleUploadedFile('test2.png', image_data2)}, instance=instance)
 >>> f.is_valid()
 True
 >>> instance = f.save()
 >>> instance.image
 <...FieldFile: tests/test2.png>
+>>> instance.height
+32
+>>> instance.width
+48
 
-# Delete the current file since this is not done by Django.
->>> instance.image.delete()
+# Delete the current file since this is not done by Django, but don't save
+# because the dimension fields are not null=True.
+>>> instance.image.delete(save=False)
 >>> instance.delete()
 
->>> f = ImageFileForm(data={'description': u'Changed it'}, files={'image': SimpleUploadedFile('test2.png', image_data)})
+>>> f = ImageFileForm(data={'description': u'Changed it'}, files={'image': SimpleUploadedFile('test2.png', image_data2)})
 >>> f.is_valid()
 True
 >>> instance = f.save()
 >>> instance.image
 <...FieldFile: tests/test2.png>
+>>> instance.height
+32
+>>> instance.width
+48
 
-# Delete the current file since this is not done by Django.
->>> instance.image.delete()
+# Delete the current file since this is not done by Django, but don't save
+# because the dimension fields are not null=True.
+>>> instance.image.delete(save=False)
 >>> instance.delete()
 
 # Test the non-required ImageField
 
->>> f = ImageFileForm(data={'description': u'Test'})
->>> f.fields['image'].required = False
+>>> class OptionalImageFileForm(ModelForm):
+...     class Meta:
+...         model = OptionalImageFile
+
+>>> f = OptionalImageFileForm(data={'description': u'Test'})
 >>> f.is_valid()
 True
 >>> instance = f.save()
 >>> instance.image
 <...FieldFile: None>
+>>> instance.width
+>>> instance.height
 
->>> f = ImageFileForm(data={'description': u'And a final one'}, files={'image': SimpleUploadedFile('test3.png', image_data)}, instance=instance)
+>>> f = OptionalImageFileForm(data={'description': u'And a final one'}, files={'image': SimpleUploadedFile('test3.png', image_data)}, instance=instance)
 >>> f.is_valid()
 True
 >>> instance = f.save()
 >>> instance.image
 <...FieldFile: tests/test3.png>
+>>> instance.width
+16
+>>> instance.height
+16
+
+# Editing the instance without re-uploading the image should not affect the image or its width/height properties
+>>> f = OptionalImageFileForm(data={'description': u'New Description'}, instance=instance)
+>>> f.is_valid()
+True
+>>> instance = f.save()
+>>> instance.description
+u'New Description'
+>>> instance.image
+<...FieldFile: tests/test3.png>
+>>> instance.width
+16
+>>> instance.height
+16
 
 # Delete the current file since this is not done by Django.
 >>> instance.image.delete()
 >>> instance.delete()
 
->>> f = ImageFileForm(data={'description': u'And a final one'}, files={'image': SimpleUploadedFile('test3.png', image_data)})
+>>> f = OptionalImageFileForm(data={'description': u'And a final one'}, files={'image': SimpleUploadedFile('test4.png', image_data2)})
 >>> f.is_valid()
 True
 >>> instance = f.save()
 >>> instance.image
-<...FieldFile: tests/test3.png>
+<...FieldFile: tests/test4.png>
+>>> instance.width
+48
+>>> instance.height
+32
 >>> instance.delete()
 
 # Test callable upload_to behavior that's dependent on the value of another field in the model
@@ -1197,6 +1330,9 @@ True
 >>> instance.image
 <...FieldFile: foo/test4.png>
 >>> instance.delete()
+"""
+
+__test__['API_TESTS'] += """
 
 # Media on a ModelForm ########################################################
 
@@ -1382,6 +1518,19 @@ ValidationError: [u'Select a valid choice. z is not one of the available choices
 >>> core.parent
 <Inventory: Pear>
 
+>>> class CategoryForm(ModelForm):
+...     description = forms.CharField()
+...     class Meta:
+...         model = Category
+...         fields = ['description', 'url']
+
+>>> CategoryForm.base_fields.keys()
+['description', 'url']
+
+>>> print CategoryForm()
+<tr><th><label for="id_description">Description:</label></th><td><input type="text" name="description" id="id_description" /></td></tr>
+<tr><th><label for="id_url">The URL:</label></th><td><input id="id_url" type="text" name="url" maxlength="40" /></td></tr>
+
 ### Validation on unique_for_date
 
 >>> p = Post.objects.create(title="Django 1.0 is released", slug="Django 1.0", subtitle="Finally", posted=datetime.date(2008, 9, 3))
@@ -1414,7 +1563,20 @@ False
 >>> f.is_valid()
 True
 
+# Model field that returns None to exclude itself with explicit fields ########
+
+>>> class CustomFieldForExclusionForm(ModelForm):
+...     class Meta:
+...         model = CustomFieldForExclusionModel
+...         fields = ['name', 'markup']
+
+>>> CustomFieldForExclusionForm.base_fields.keys()
+['name']
+
+>>> print CustomFieldForExclusionForm()
+<tr><th><label for="id_name">Name:</label></th><td><input id="id_name" type="text" name="name" maxlength="10" /></td></tr>
+
 # Clean up
 >>> import shutil
 >>> shutil.rmtree(temp_storage_dir)
-"""}
+"""
