@@ -271,10 +271,8 @@ blank=True for the model field.  Finally, the widget should have the
 ...     form = AdminConcertForm
 
 >>> ma = ConcertAdmin(Concert, site)
->>> ks = ma.get_form(request).base_fields.keys()
->>> ks.sort()
->>> ks
-['day', 'main_band', 'opening_band']
+>>> ma.get_form(request).base_fields.keys()
+['main_band', 'opening_band', 'day']
 
 >>> class AdminConcertForm(forms.ModelForm):
 ...     extra = forms.CharField()
@@ -286,15 +284,14 @@ blank=True for the model field.  Finally, the widget should have the
 ...     form = AdminConcertForm
 
 >>> ma = ConcertAdmin(Concert, site)
->>> ks = ma.get_form(request).base_fields.keys()
->>> ks.sort()
->>> ks
+>>> ma.get_form(request).base_fields.keys()
 ['extra', 'transport']
 
 >>> class ConcertInline(TabularInline):
 ...     form = AdminConcertForm
 ...     model = Concert
 ...     fk_name = 'main_band'
+...     can_delete = True
 
 >>> class BandAdmin(ModelAdmin):
 ...     inlines = [
@@ -302,10 +299,9 @@ blank=True for the model field.  Finally, the widget should have the
 ...     ]
 
 >>> ma = BandAdmin(Band, site)
->>> ks = list(ma.get_formsets(request))[0]().forms[0].fields.keys()
->>> ks.sort()
->>> ks
-['DELETE', 'extra', 'id', 'main_band', 'transport']
+>>> list(ma.get_formsets(request))[0]().forms[0].fields.keys()
+['extra', 'transport', 'id', 'DELETE', 'main_band']
+
 
 >>> band.delete()
 
